@@ -282,7 +282,7 @@ FORMAT_REC fecommon_core_formats[] = {
 	{ "glib_error", "{error $0} $1", 2, { 0, 0 } },
 	{ "overwrite_config", "Overwrite config (y/N)?", 0 },
 	{ "set_title", "[{hilight $0}]", 1, { 0 } },
-	{ "set_item", "$0 = $1", 2, { 0, 0 } },
+	{ "set_item", "$[-!32]0 %_$1", 2, { 0, 0 } },
 	{ "set_unknown", "Unknown setting $0", 1, { 0 } },
 	{ "set_not_boolean", "Setting {hilight $0} isn't boolean, use /SET", 1, { 0 } },
 	{ "no_completions", "There are no completions", 0 },
