@@ -41,8 +41,8 @@ FORMAT_REC gui_text_formats[] =
 	{ "refnum_not_found", "Window number $0 not found", 1, { 0 } },
 	{ "window_too_small", "Not enough room to resize this window", 0 },
 	{ "cant_hide_last", "You can't hide the last window", 0 },
-	{ "cant_hide_sticky_windows", "You can't hide sticky windows (use /WINDOW STICK OFF)", 0 },
-	{ "cant_show_sticky_windows", "You can't show sticky windows (use /WINDOW STICK OFF)", 0 },
+	{ "cant_hide_sticky_windows", "You can't hide sticky windows (use /SET autounstick_windows ON)", 0 },
+	{ "cant_show_sticky_windows", "You can't show sticky windows (use /SET autounstick_windows ON)", 0 },
 	{ "window_not_sticky", "Window is not sticky", 0 },
 	{ "window_set_sticky", "Window set sticky", 0 },
 	{ "window_unset_sticky", "Window is not sticky anymore", 0 },
@@ -67,6 +67,7 @@ FORMAT_REC gui_text_formats[] =
 	{ "statusbar_info_item_footer", "", 0 },
 	{ "statusbar_info_item_name",  "%#         : $[35]0 $[9]1 $2", 3, { 0, 1, 0 } },
 	{ "statusbar_not_found", "Statusbar doesn't exist: $0", 1, { 0 } },
+	{ "statusbar_not_found", "Statusbar is disabled: $0", 1, { 0 } },
 	{ "statusbar_item_not_found", "Statusbar item doesn't exist: $0", 1, { 0 } },
 	{ "statusbar_unknown_command", "Unknown statusbar command: $0", 1, { 0 } },
 	{ "statusbar_unknown_type", "Statusbar type must be 'window' or 'root'", 1, { 0 } },
@@ -87,7 +88,7 @@ FORMAT_REC gui_text_formats[] =
 	  "|_ _|_ _ _____(_)%:"
 	  " | || '_(_-<_-< |%:"
 	  "|___|_| /__/__/_|%:"
-	  "Irssi v$J - http://www.irssi.org", 0 },
+	  "Irssi v$J - https://irssi.org", 0 },
 	{ "welcome_firsttime",
 	  "- - - - - - - - - - - - - - - - - - - - - - - - - - - -\n"
 	  "Hi there! If this is your first time using Irssi, you%:"
